/**
  EasyVR 
  SonicNet Demo (Bidirectional)

  Simple chat demo showing the SonicNet communication features of the EasyVR 3.
  
  The code is compatible with older EasyVR 2.0 modules.

  The module sends a sonic token while saying "Is there anybody out there?"
  and waits for a reply from a second module with the same firmware.
  If the reply arrives, the two modules start giving commands to each other
  (e.g. "Please switch ON your LED", "Please move your servo" with embedded
  SonicNet tokens).
  
  The servo is connected to pin 9 and the LED used is the one connected
  to pin IO1 of EasyVR.

  >>>*** In order for the demo to work, you have to turn on the second module
  after the first one said "Is there anybody out there?" ***<<<

**
  Example code for the EasyVR library v1.8
  Written in 2016 by RoboTech srl for VeeaR <http:://www.veear.eu>

  To the extent possible under law, the author(s) have dedicated all
  copyright and related and neighboring rights to this software to the
  public domain worldwide. This software is distributed without any warranty.

  You should have received a copy of the CC0 Public Domain Dedication
  along with this software.
  If not, see <http://creativecommons.org/publicdomain/zero/1.0/>.
*/

#include "Arduino.h"
#if !defined(SERIAL_PORT_MONITOR)
  #error "Arduino version not supported. Please update your IDE to the latest version."
#endif

#if defined(SERIAL_PORT_USBVIRTUAL)
  // Shield Jumper on HW (for Leonardo and Due)
  #define port SERIAL_PORT_HARDWARE
  #define pcSerial SERIAL_PORT_USBVIRTUAL
#else
  // Shield Jumper on SW (using pins 12/13 or 8/9 as RX/TX)
  #include "SoftwareSerial.h"
  SoftwareSerial port(12, 13);
  #define pcSerial SERIAL_PORT_MONITOR
#endif

#include "EasyVR.h"

EasyVR easyvr(port);

// Exported by EasyVR Commander
#include "EasyVR_custom_data.h"

#include <Servo.h>
Servo myservo;

int8_t bits = 4;
int8_t servo_pos = 2;
int8_t idx = 0;


void setup()
{
  // setup PC serial port
  pcSerial.begin(9600);

  // bridge mode?
  int mode = easyvr.bridgeRequested(pcSerial);
  switch (mode)
  {
  case EasyVR::BRIDGE_NONE:
    // setup EasyVR serial port
    port.begin(9600);
    // run normally
    pcSerial.println(F("---"));
    pcSerial.println(F("Bridge not started!"));
    break;

  case EasyVR::BRIDGE_NORMAL:
    // setup EasyVR serial port (low speed)
    port.begin(9600);
    // soft-connect the two serial ports (PC and EasyVR)
    easyvr.bridgeLoop(pcSerial);
    // resume normally if aborted
    pcSerial.println(F("---"));
    pcSerial.println(F("Bridge connection aborted!"));
    break;

  case EasyVR::BRIDGE_BOOT:
    // setup EasyVR serial port (high speed)
    port.begin(115200);
    // soft-connect the two serial ports (PC and EasyVR)
    easyvr.bridgeLoop(pcSerial);
    // resume normally if aborted
    pcSerial.println(F("---"));
    pcSerial.println(F("Bridge connection aborted!"));
    break;
  }
  
  myservo.attach(9);
  myservo.write(servo_pos*40);

  if (!easyvr.detect())
  {
    Serial.println("EasyVR not detected!");
    for (;;);
  }

  easyvr.setPinOutput(EasyVR::IO1, LOW);
  Serial.print("EasyVR detected, version ");
  Serial.println(easyvr.getID());

  easyvr.embedToken(bits, 0, 1);
  easyvr.playSound(SND_is_there_anybody , EasyVR::VOL_FULL);  // looking for other modules...
}

void loop()
{
  easyvr.detectToken(bits, EasyVR::REJECTION_AVG, 0);
  while (!easyvr.hasFinished()); // waits for another module reply
  idx = easyvr.getToken();
  switch (idx)
  {
  case 0: //reply received, I'm the master!
    Serial.println("Mate found!");
    delay(1500);
    easyvr.playSound(SND_I_found_you , EasyVR::VOL_FULL);
    for (idx = 1; idx < 5; ++idx)
    {
      delay(1500);
      easyvr.embedToken(bits, idx, 1);
      easyvr.playSound(idx , EasyVR::VOL_FULL);  // giving command to the other module...
      easyvr.detectToken(bits, EasyVR::REJECTION_AVG, 0);
      while (!easyvr.hasFinished()); // waits for ack without checking token number
    }
    break;

  case 1: // Slave: move servo down
    Serial.println("Servo down");
    delay(2000);
    if (servo_pos > 1) servo_pos--;
    myservo.write(servo_pos*40);
    easyvr.embedToken(bits, 0, 1);
    easyvr.playSound(SND_done , EasyVR::VOL_FULL);  // Sending ACK
    break;

  case 2: // Slave: move servo up
    Serial.println("Servo up");
    delay(2000);
    if (servo_pos < 4) servo_pos++;
    myservo.write(servo_pos*40);
    easyvr.embedToken(bits, 0, 1);
    easyvr.playSound(SND_done , EasyVR::VOL_FULL);  // Sending ACK
    break;

  case 3: // Slave: LED ON
    Serial.println("LED ON");
    delay(2000);
    easyvr.setPinOutput(EasyVR::IO1, HIGH);
    easyvr.embedToken(bits, 0, 1);
    easyvr.playSound(SND_done , EasyVR::VOL_FULL);  // Sending ACK
    break;

  case 4: // Slave: LED OFF
    Serial.println("LED OFF");
    delay(2000);
    easyvr.setPinOutput(EasyVR::IO1, LOW);
    easyvr.embedToken(bits, 0, 1);
    easyvr.playSound(SND_done , EasyVR::VOL_FULL);  // Sending ACK
    break;
  }
}



